@@ -3,20 +3,19 @@
 print.emfrail <- function(x, ...) {
 
   obj <- x
-  cat("Call: \n",
-      "emfrail(",
-      format(obj$.formula),
-      ")\n",
-      sep = "")
+  cat("Call: \n")
+  dput(attr(obj, "call"))
+  cat("\n")
+  # names(obj$outer_m)[1] <- "theta"
+  # obj$outer_m[1] <- exp(obj$outer_m[1])
+  #
+  # names(obj$outer_m)[2] <- "loglik"
+  # obj$outer_m[2] <- (-1) * obj$outer_m[2]
 
-  names(obj$outer_m)[1] <- "theta"
-  obj$outer_m[1] <- exp(obj$outer_m[1])
-
-  names(obj$outer_m)[2] <- "loglik"
-  obj$outer_m[2] <- (-1) * obj$outer_m[2]
-
-  print(obj$outer_m)
+  # print(obj$outer_m)
 
+  cat("log-likelihood:", -obj$outer_m$objective, "\n")
+  cat("theta:", exp(obj$outer_m$minimum), "\n")
   cat("\n")
   if(length(obj$inner_m$coef) > 0) {
     coefmat <- list(
@@ -38,6 +37,11 @@ print.emfrail <- function(x, ...) {
     cat("Commenges-Andersen test for heterogeneity: p-val", format(obj$ca_test[3], digits = 3))
   }
 
+  if(!is.null(obj$cens_test)) {
+    cat("\n")
+    cat("Score test for dependent censoring: p-val", format(obj$cens_test[2], digits = 3))
+  }
+
   invisible(obj)
 
 }
